      *
      * Module:         ECIB1.CBL
      *
      * Description:    Basic CICS TG Client API Cobol Sample using the
      *                 External Call Interface
      *
      * Prerequisites:  For successful operation this samples requires
      *                 the CICS program EC01 installed on your CICS
      *                 server. The source for EC01 is installed as
      *                 <install_path>/samples/server/ec01.ccp
      *
      *                 Before compiling with Microfocus Cobol this
      *                 sample program needs to be modified, the
      *                 instructions on what to edit are included
      *                 in the program comments.
      *
      * NOTE:
      *
      * The following code is sample code created by IBM Corporation.
      * This sample code is not part of any standard IBM product and
      * is provided to you solely for the purpose of assisting you in
      * the development of your applications.  The code is provided
      * 'AS IS', without warranty or condition of any kind.  IBM shall
      * not be liable for any damages arising out of your use of the
      * sample code, even if IBM has been advised of the possibility
      * of such damages.
      *
      * Licensed Materials - Property of IBM  
      *  
      * 5724-I81,5725-B65 
      *  
      * (C) Copyright IBM Corp. 1994, 2014 All Rights Reserved.  
      *  
      * US Government Users Restricted Rights - Use, duplication or  
      * disclosure restricted by GSA ADP Schedule Contract with  
      * IBM Corp.  
      *
      * Status: Version 9 Release 2 
      *

       IDENTIFICATION DIVISION.
       PROGRAM-ID.    ECIB1.

       ENVIRONMENT DIVISION.
       CONFIGURATION SECTION.
       SOURCE-COMPUTER. IBM.
       OBJECT-COMPUTER. IBM.
      * To compile for Microfocus Cobol uncomment the following
      * four lines.
      *SPECIAL-NAMES.
      *    CALL-CONVENTION 8 IS CICS
      *    SYSIN   IS CRT1
      *    CONSOLE IS CRT2.

       DATA DIVISION.
       WORKING-STORAGE SECTION.
      * Working storage starts here

       01  FILLER                            PIC X(32)  VALUE
           '** Working storage starts here**'.

       01  FILLER                           PIC X(8)   VALUE 'WS-'.
       01  WS-DEBUG-AREA.
           05 WS-PROG-NAME                  PIC X(16) VALUE SPACES.
           05 WS-PROG-SECT-NAME             PIC X(32) VALUE SPACES.
           05 WS-ERROR-SUBSCRIPT            PIC S9(4) COMP-5.
           05 WS-IN-EIBCALEN                PIC 9(8) COMP VALUE 80.
           05 WS-EIBRESP                    PIC S9(8) COMP.
           05 WS-EIBRESP-DISP               PIC S9(9)
              SIGN LEADING SEPARATE.
           05  WS-DEBUG-ON-FLAG             PIC X VALUE 'N'.
               88  DEBUG-ON                 VALUE 'Y'.

      * Variable definitions
       01 WORKING-VARIABLES.
           02 NULL-PTR              POINTER.
      * Used by ECIListSystems
           02 NAMESPACE             POINTER.
           02 SYSTEMS               PIC 9(4)  COMP-5.
           02 LOOP-COUNT            PIC 9(1)  COMP-5.
           02 WS-LOOP-COUNT         PIC 9(1).

       01 WS-AREA.
           02 COMMAREA                   PIC X(18) VALUE LOW-VALUES.

      * Fill in your server here
           02 SERVER                     PIC X(8)  VALUE LOW-VALUES.

      * Fill in your user id here
      *    02 USERID                     PIC X(8)  VALUE LOW-VALUES.
           02 USERID                     PIC X(8)  VALUE 'SYSAD   '.

      * Fill in your password here
      *    02 PASSWD                     PIC X(8)  VALUE LOW-VALUES.
           02 PASSWD                     PIC X(8)  VALUE 'SYSAD   '.

           02 WS-CONNECTION-TEXT.
               05 FILLER                 PIC X(21)
                   VALUE 'Connection Type : '.
               05 FILLER                 PIC X(24).
                   88 WS-ECI-CONNECTED-NOWHERE
                       VALUE 'ECI_CONNECTED_NOWHERE'.
                   88 WS-ECI-CONNECTED-TO-SERVER
                       VALUE 'ECI_CONNECTED_TO_SERVER'.
                   88 WS-ECI-CONNECTED-TO-CLIENT
                       VALUE 'ECI_CONNECTED_TO_CLIENT'.
                   88 WS-ECI-CONNECTED-UNKNOWN
                       VALUE 'Unknown'.
           02 WS-SERVERSTATUS-TEXT.
               05 FILLER                 PIC X(21)
                  VALUE 'Server status   : '.
               05 FILLER                 PIC X(21).
                   88 WS-ECI-SERVERSTATE-UP
                       VALUE 'ECI_SERVERSTATE_UP'.
                   88 WS-ECI-SERVERSTATE-DOWN
                       VALUE 'ECI_SERVERSTATE_DOWN'.
                   88 WS-ECI-SERVERSTATE-UNKNOWN
                       VALUE 'Unknown'.
           02 WS-CLIENTSTATUS-TEXT.
               05 FILLER                 PIC X(21)
                   VALUE 'Client status   : '.
               05 FILLER                 PIC X(29).
                   88 WS-ECI-CLIENTSTATE-UP
                       VALUE 'ECI_CLIENTSTATE_UP'.
                   88 WS-ECI-CLIENTSTATE-INAPPL
                       VALUE 'ECI_CLIENTSTATE_INAPPLICABLE'.
                   88 WS-ECI-CLIENTSTATE-UNKNOWN
                       VALUE 'Unknown'.
      * Used by Response
       02  RESPONSE-TEXT PIC X(30) VALUE 'CICS-ECI-NORMAL'.
       02  NAME-TEXT     PIC X(30) VALUE SPACES.
       02  WS-CICS-ECI-ERRORS.
      *    Cobol starts subscripting from 1 so return-code(0)
      *    i.e. ECI-NO-ERROR is not included in this array so as to
      *    keep this array in step with the error numbers in CICSECI.CBL
      *    05  FILLER PIC X(30) VALUE 'ECI-NO-ERROR                  '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-INVALID-DATA-LENGTH   '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-INVALID-EXTEND-MODE   '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-NO-CICS               '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-CICS-DIED             '.
      *    05  FILLER PIC X(30) VALUE 'ECI-ERR-REQUEST-TIMEOUT       '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-NO-REPLY              '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-RESPONSE-TIMEOUT      '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-TRANSACTION-ABEND     '.
      *    05  FILLER PIC X(30) VALUE 'ECI-ERR-EXEC-NOT-RESIDENT     '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-LUW-TOKEN             '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-SYSTEM-ERROR          '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-NULL-WIN-HANDLE       '.
           05  FILLER PIC X(30) VALUE 'ECI-ERROR-ID = -11 (Unused)   '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-NULL-MESSAGE-ID       '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-THREAD-CREATE-ERROR   '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-INVALID-CALL-TYPE     '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-ALREADY-ACTIVE        '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-RESOURCE-SHORTAGE     '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-NO-SESSIONS           '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-NULL-SEM-HANDLE       '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-INVALID-DATA-AREA     '.
           05  FILLER PIC X(30) VALUE 'ECI-ERROR-ID = -21 (Unused)   '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-INVALID-VERSION       '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-UNKNOWN-SERVER        '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-CALL-FROM-CALLBACK    '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-INVALID-TRANSID       '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-MORE-SYSTEMS          '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-NO-SYSTEMS            '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-SECURITY-ERROR        '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-MAX-SYSTEMS           '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-MAX-SESSIONS          '.
           05  FILLER PIC X(30) VALUE 'ECI-ERR-ROLLEDBACK            '.

       02  WS-TBL-CICS-ECI-ERRORS REDEFINES WS-CICS-ECI-ERRORS.
           05  WS-CICS-ECI-ERROR-LINE OCCURS 30 TIMES.
               10  WS-CICS-ECI-ERROR  PIC X(30).
      * CICS ECI API variable definitions                           *
           COPY "cicseci.cbl".

        PROCEDURE DIVISION.

       A-CONTROL SECTION.
           DISPLAY 'Starting the Basic ECI COBOL Sample'.
           DISPLAY ' '.
           PERFORM B-LIST-SYSTEMS.
           IF DEBUG-ON
               PERFORM C-STATE-SYNC
           END-IF.
           PERFORM D-SYNC-CALL.
           DISPLAY ' '
           DISPLAY 'ECI Sample Program Completed'
           GOBACK.

       A-EXIT.
           EXIT.

      *
       B-LIST-SYSTEMS SECTION.
      * Issue a CICS_ECIListSystems call


           DISPLAY 'Servers Defined in Client:'.
      * Ask the Client for a list of servers to connect to.
           SET NAMESPACE TO NULL.
           MOVE 1  TO CICS-ECINUMSYS.
           MOVE 9  TO SYSTEMS.

      * To compile for Microfocus Cobol uncomment the next line and
      * comment out the line that follows.
      *    CALL CICS 'CICSECILISTSYSTEMS'
           CALL 'CICSECILISTSYSTEMS'
                USING BY REFERENCE NAMESPACE
                      BY REFERENCE SYSTEMS
                      BY REFERENCE CICS-ECISYSTEM
                RETURNING ECI-ERROR-ID.

           MOVE 'CICS_ECIListSystems:' TO NAME-TEXT.

           IF ECI-NO-ERROR OR ECI-ERR-MORE-SYSTEMS
                 PERFORM WITH TEST BEFORE
                         VARYING LOOP-COUNT FROM 1 BY 1
                         UNTIL LOOP-COUNT > SYSTEMS
                            OR LOOP-COUNT > 9
                            OR SYSTEMNAME(LOOP-COUNT) <= SPACES
                     MOVE LOOP-COUNT TO WS-LOOP-COUNT
                     DISPLAY WS-LOOP-COUNT
                             '  '
                             SYSTEMNAME(LOOP-COUNT)
                             '       '
                             SYSTEMDESC(LOOP-COUNT)
                 END-PERFORM
           END-IF.

           PERFORM RESPONSE.

           DISPLAY 'Choose Server to connect to: '
           ACCEPT WS-LOOP-COUNT.
           MOVE WS-LOOP-COUNT TO LOOP-COUNT.
           MOVE LOOP-COUNT TO WS-LOOP-COUNT.
           MOVE SYSTEMNAME(LOOP-COUNT) TO SERVER.
           DISPLAY SERVER.
           DISPLAY 'Enter your Userid:'.
           ACCEPT  USERID.
           DISPLAY 'Enter Password:'.
           ACCEPT  PASSWD.

        B-EXIT.
           EXIT.


      *
        C-STATE-SYNC SECTION.
      *
      * Issue a CICS_Externalcall for an ECI_STATE_SYNC
      *

           DISPLAY 'ECI_STATE_SYNC test'
           MOVE LOW-VALUES           TO ECI-PARMS
                                        ECI-STATUS.
           SET  ECI-STATE-SYNC       TO TRUE.
           MOVE USERID               TO ECI-USERID.
           MOVE PASSWD               TO ECI-PASSWORD.
           MOVE SERVER               TO ECI-SYSTEM-NAME.
           SET  ECI-COMMAREA         TO ADDRESS OF ECI-STATUS.
           MOVE LENGTH OF ECI-STATUS TO ECI-COMMAREA-LENGTH.
           MOVE 0                    TO ECI-TIMEOUT.
           SET  ECI-STATE-IMMEDIATE  TO TRUE.
           SET  ECI-NO-ERROR         TO TRUE.
           SET  ECI-VERSION-1A       TO TRUE.

      * Make the ECI call.
      * To compile for Microfocus Cobol uncomment the next line and
      * comment out the line that follows.
      *    CALL CICS 'CICSEXTERNALCALL'
           CALL 'CICSEXTERNALCALL'
                USING BY REFERENCE ECI-PARMS
                RETURNING ECI-ERROR-ID.

           IF ECI-NO-ERROR

      * Test the ECI connection type.
             EVALUATE TRUE
                 WHEN ECI-CONNECTED-NOWHERE
                     SET WS-ECI-CONNECTED-NOWHERE   TO TRUE
                 WHEN ECI-CONNECTED-TO-SERVER
                     SET WS-ECI-CONNECTED-TO-SERVER TO TRUE
                 WHEN ECI-CONNECTED-TO-CLIENT
                     SET WS-ECI-CONNECTED-TO-CLIENT TO TRUE
                 WHEN OTHER
                     SET WS-ECI-CONNECTED-UNKNOWN   TO TRUE
             END-EVALUATE
             DISPLAY WS-CONNECTION-TEXT

      * Test the ECI server status.
             EVALUATE TRUE
                 WHEN ECI-SERVERSTATE-UP
                      SET WS-ECI-SERVERSTATE-UP      TO TRUE
                 WHEN ECI-SERVERSTATE-DOWN
                      SET WS-ECI-SERVERSTATE-DOWN    TO TRUE
                 WHEN OTHER
                      SET WS-ECI-SERVERSTATE-UNKNOWN TO TRUE
             END-EVALUATE
             DISPLAY WS-SERVERSTATUS-TEXT

      * Test the ECI requester status.
             EVALUATE TRUE
                 WHEN ECI-CLIENTSTATE-UNKNOWN
                      SET WS-ECI-CLIENTSTATE-UNKNOWN TO TRUE
                 WHEN ECI-CLIENTSTATE-UP
                      SET WS-ECI-CLIENTSTATE-UP      TO TRUE
                 WHEN ECI-CLIENTSTATE-INAPPLICABLE
                      SET WS-ECI-CLIENTSTATE-INAPPL  TO TRUE
                 WHEN OTHER
                      SET WS-ECI-CLIENTSTATE-UNKNOWN TO TRUE
             END-EVALUATE
             DISPLAY WS-CLIENTSTATUS-TEXT

           END-IF.

           MOVE 'ECI_STATE_SYNC:' TO NAME-TEXT.
           PERFORM RESPONSE.

        C-EXIT.
           EXIT.

      *
        D-SYNC-CALL SECTION.
      *
      * Issue a CICS_Externalcall for an ECI_SYNC
      *

           MOVE LOW-VALUES TO ECI-PARMS

           SET  ECI-SYNC            TO TRUE
           MOVE 'EC01'              TO ECI-PROGRAM-NAME
           MOVE USERID              TO ECI-USERID
           MOVE PASSWD              TO ECI-PASSWORD
           MOVE SERVER              TO ECI-SYSTEM-NAME
           SET  ECI-COMMAREA        TO ADDRESS OF COMMAREA
           MOVE LENGTH OF COMMAREA  TO ECI-COMMAREA-LENGTH
           MOVE 0                   TO ECI-TIMEOUT
           SET  ECI-NO-EXTEND       TO TRUE
           SET  ECI-VERSION-1A      TO TRUE


           DISPLAY 'ECI_SYNC test'
      * To compile for Microfocus Cobol uncomment the next line and
      * comment out the line that follows.
      *    CALL CICS 'CICSEXTERNALCALL'
           CALL 'CICSEXTERNALCALL'
                   USING BY REFERENCE ECI-PARMS
                RETURNING ECI-ERROR-ID.

           IF ECI-NO-ERROR
              DISPLAY 'Transaction EC01 returned with CommArea:'
                      COMMAREA
           END-IF

           MOVE 'ECI_SYNC:' TO NAME-TEXT
           PERFORM RESPONSE

           EXIT.

      *
      * Response
      *
      * Display the immediate response code from an ECI call
      *
       RESPONSE SECTION.

           IF NOT (ECI-NO-ERROR OR ECI-ERR-MORE-SYSTEMS)
               MULTIPLY ECI-ERROR-ID BY -1 GIVING WS-ERROR-SUBSCRIPT
               MOVE WS-CICS-ECI-ERROR(WS-ERROR-SUBSCRIPT)
                 TO RESPONSE-TEXT
               DISPLAY 'Error: ' NAME-TEXT '  '
                       ECI-ERROR-ID ' ' RESPONSE-TEXT
               IF ECI-ERR-TRANSACTION-ABEND
                   DISPLAY 'Abend code was: ' ECI-ABEND-CODE
               END-IF
           END-IF.

           IF ECI-ERR-SYSTEM-ERROR
              PERFORM DO-ERROR
           END-IF.

           EXIT.

      *
      * Display the System Error response from an ECI Call
      *
       DO-ERROR SECTION.

           DISPLAY  'System Error:'
           DISPLAY  'Return Code:' ECI-ERROR-ID

           PERFORM DO-EXIT

           EXIT.

      *
      * DO-EXIT
      *
      * End program
      *
       DO-EXIT SECTION.

           GOBACK.

           EXIT.
      *end of program
