/*               CICS TG Client API Classes

   FILE NAME:    cclcon.hpp

   CLASS:        CclConn

   DESCRIPTION:  Class to control a server connection

 Licensed Materials - Property of IBM  
  
 "Restricted Materials of IBM"  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1995, 2014  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 2 
*/


#ifndef _CCLCON_
#define _CCLCON_

class CclConn {
friend class CclOConn;
public:

// enumeration

  enum           ServerStatus { unknown,available,unavailable };


// constructors/destructors

  CCLI           CclConn(
                   const char* serverName = 0,    //server name, if not default
                   const char* userId = 0,                //userid, if required
                   const char* password = 0,            //password, if required
                   const char* runTran = 0,         //tran to run under, if any
                   const char* attachTran = 0 );       //tran to attach, if any
  CCLI virtual  ~CclConn();


// actions

  CCLI void      cancel(                               //cancel changed request
                   CclFlow& flow );                               //user's flow
  CCLI void      changed(           //request server status change notification
                   CclFlow& flow );                               //user's flow
  CCLI void      link(                                    //call server program
                   CclFlow& flow,                                 //user's flow
                   const char* programName,                      //program name
                   CclBuf* commarea = 0,            //commarea, if passing data
                   CclUOW* unit = 0 );           //UOW, if a participating call
  CCLI void      status(                                //request server status
                   CclFlow& flow );                               //user's flow
  CCLI void      alterSecurity(                       // update userid/password
                   const char* newUserid,
                   const char* newPassword);
  CCLI void      makeSecurityDefault(); // set current usrid/pw to client deflt
  CCLI CclSecAttr* verifyPassword();        // verify userid/password at server
  CCLI CclSecAttr* changePassword(
                     const char* newPassword);     // change password at server


// attributes

  CCLI const char*
                 listState() const;                                //state text
  CCLI const char*
                 password() const;
  CCLI const char*
                 serverName() const;
  CCLI ServerStatus
                 serverStatus() const;
  CCLI const char*
                 serverStatusText() const;
  CCLI const char*
                 userId() const;
  CCLI const char*
                 password(Ccl::Bool unpadded) const;
  CCLI const char*
                 userId(Ccl::Bool unpadded) const;
  CCLI const char*
                 serverName(Ccl::Bool unpadded) const;

private:
                 CclConn( const CclConn& );         //disallow copy constructor
  CclConn&       operator=( const CclConn& );             //disallow assignment
  CclConnI*      ivpI;                                                //private
};

#endif
