/*              CICS TG Client API Classes

   FILE NAME:   cclepi.hpp

   DESCRIPTION: C++ declaration of CICS TG Client API EPI Class (CclEPI)

 Licensed Materials - Property of IBM  
  
 "Restricted Materials of IBM"  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1995, 2014  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 2 
*/


#ifndef _CCLEPI_
#define _CCLEPI_

class CclEPI {
public:

// Typedefs and enumerations

  enum           State {
                   active, discon, error };


// Constructors/Destructors

  CCLI           CclEPI();
  CCLI virtual   ~CclEPI();


// Actions

  CCLI virtual void
                 handleException(      // exception handler (overridable)
                   CclException exception );
  CCLI void      terminate();          // terminate EPI


// Accessors

  CCLI unsigned short
                 serverCount();        // get number of CICS server names
  CCLI const char*
                 serverName(           // get specified CICS server name
                   unsigned short index=1 );
  CCLI const char*
                 serverDesc(           // get specified CICS server description
                   unsigned short index=1 );

  CCLI State     state() const;        // get object state
  CCLI Ccl::ExCode exCode() const;     // get object exception code
  CCLI const char*
                 diagnose();           // get description of current exception
  CCLI long      errorCause() const;   // additional error code - cause
  CCLI long      errorValue() const;   // additional error code - value
  CCLI const char*                     // most recent exception code
                 exCodeText() const;   //   - text

private:
  CclEPII*       ivpI;                 // pointer to implementation class

}; // end of class CclEPI

#endif
