/*               CICS TG Client API Classes

   FILE NAME:    cclexc.hpp

   CLASS:        CclException

   DESCRIPTION:  Class used during exception handling

 Licensed Materials - Property of IBM  
  
 "Restricted Materials of IBM"  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1995, 2014  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 2 
*/

#ifndef _CCLEXC_
#define _CCLEXC_

class CclException {
public:

// constructors/destructors

  CCLI           CclException(
                   unsigned short originCode,                     //origin code
                   Ccl::ExCode exCode,                         //exception code
                   const char* className,                          //class name
                   const char* methodName,                        //method name
                   void* exObject = 0,               //exception object, if any
                   const char* abendCode = 0);
  CCLI           CclException(                               //copy constructor
                   const CclException& except );       //exception to be copied
  CCLI virtual  ~CclException();


// attributes

  CCLI const char*
                 className() const;                                //class name
  CCLI const char*
                 diagnose() const;                            //diagnostic text
  CCLI Ccl::ExCode
                 exCode() const;                               //exception code
  CCLI const char*
                 exCodeText() const;                     //exception code..text
  CCLI void*     exObject() const;                   //exception object, if any
  CCLI const char*
                 methodName() const;                              //method name
  CCLI unsigned short
                 originCode() const;                              //origin code
  CCLI const char*
                 abendCode() const;

private:
  CclException&  operator=( const CclException& );        //disallow assignment
  CclExceptionI* ivpI;                                                //private
};

#endif
