/*              CICS TG Client API Classes

   FILE NAME:   cclmap.hpp

   DESCRIPTION: C++ declaration of CICS TG Client API EPI Map Class (CclMap)

 Licensed Materials - Property of IBM  
  
 "Restricted Materials of IBM"  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1995, 2014  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 2 
*/


#ifndef _CCLMAP_
#define _CCLMAP_

class CclMap {
public:

// Constructors/Destructors

  CCLI           CclMap(
                    CclScreen* screen );         // current screen
  CCLI           ~CclMap();


// Accessors

  CCLI CclField* field(                // access CclField object by index
                   unsigned short index );       // index in BMS map
  CCLI CclField* field(                // access CclField object by position
                   unsigned short row,           // row in BMS map
                   unsigned short column );      // column in BMS map

  CCLI Ccl::ExCode exCode() const;     // get current exception code


// Typedefs for derived class data members

  typedef struct { unsigned short row;       // Map row position on screen
                   unsigned short col;       // Map column position on screen
                   unsigned short width;     // Map width in columns
                   unsigned short depth;     // Map depth in rows
                   unsigned short fields;    // Number of fields
                   unsigned short labels;    // Number of labeled fields
                 } MapData;

  typedef unsigned short
                 FieldIndex;

  typedef struct { unsigned short row; // Field row (within map)
                   unsigned short col; // Field column (within map)
                   unsigned short len; // Field length
                 } FieldData;

protected:

// Actions

  CCLI void      validate(             // validate map against current screen
                   const MapData* map,
                   const FieldIndex* index,
                   const FieldData* fields);
  CCLI CclField* namedField(           // get named BMS field in map
                   unsigned long name );

// Data members

  Ccl::ExCode    ivExCode;             // exception code
  CclScreen*     ivpScreen;            // pointer to CclScreen object
  const MapData*
                 ivMapData;            // map data array
  const FieldIndex*
                 ivFieldIndex;         // field index array
  const FieldData*
                 ivFieldData;          // field data array

}; // end of class CclMap

#endif
