/* File Name(s)  : ecib1.cpp

   Product       : CICS Transaction Gateway

   Description   : This is a basic sample showing how to connect to a CICS
                   server using the CICS Client API. It allows the user to
                   choose a server to connect to from those available and
                   then starts a transaction on that server.

   Prerequisites:  This sample requires the CICS Program EC01 to be
                   installed on your CICS server for operation.

 Licensed Materials - Property of IBM  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1999, 2014 All Rights Reserved.  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 2 

                   The following code is sample code created by IBM
                   Corporation.  This sample code is not part of any
                   standard IBM product and is provided to you solely for
                   the purpose of assisting you in the development of your
                   applications.  The code is provided 'AS IS', without
                   warranty or condition of any kind.  IBM shall not be
                   liable for any damages arising out of your use of the
                   sample code, even if IBM has been advised of the
                   possibility of such damages. */


// Include files

#include <string.h>

#if defined(CICS_W32) || defined(CICS_LNX)
#include <iostream>
#include <iomanip>
using namespace std;
#else
#include <iostream.h>
#endif

#include <math.h>
#include <stdlib.h>

#include <cicseci.hpp>

#define copyright_notice copyright4cppecib1
char version4cppecib1[] =
"\nCICS Transaction Gateway"
" Version 9.2.0.3"
" Build Level c920-20180312-1735" ;

char copyright_notice[] =
"\n"
"Licensed Materials - Property of IBM. "
"\n"
"5724-I81,5725-B65 "
"\n"
"(c) Copyright IBM Corp. 1999, 2014 All Rights Reserved. "
"\n"
"US Government Users Restricted Rights - Use, duplication or "
"disclosure restricted by GSA ADP Schedule Contract with "
"IBM Corp."
"\n";

// Defines

#define ECI_DATA_LENGTH 18			     //Set the length of your COMMAREA here.
#define CHOICE_LENGTH	100				 //Set the length of the response for server the choice


// The main entrypoint

int main(void){
   char   chosenServer[8 +1];         //Server name of users choice.
   char   progid[4 +1] = "EC01";      //Program to be executed.
   CclBuf commarea("");               //COMMAREA for use in transactions.
   char  *reply;

   char   UserID[9] = "sysad";        //Enter your user ID here.
   char   PassWd[9] = "sysad";        //Enter your password here.

   int    i, loop=1;
   char  choice[CHOICE_LENGTH];		  //Stores the server choice made by user


   // Display a banner

   cout << "CICS Transaction Gateway Basic C++ ECI V1 Sample" << endl << endl;


   // Open try block to catch all Class exceptions.

   try {


      // Retrieve list of servers by creating new instance of CclEci

      CclECI *pEci = CclECI::instance();


      //Get user choice for server to be used

      do {


         // Display server choices

         cout << "Servers Available in Client daemon:" << endl;
         for (i=1;i<=pEci->serverCount();i++) {
            cout << "    " << i << "." << pEci->serverName(i) << endl;
         }


         // Ask user to chose the server to call

         cout << endl << "Choose Server to connect to:";
         cout.flush();
         if (cin.peek() == '\n') cin.ignore(1, '\n');
         cin.getline(choice, CHOICE_LENGTH, '\n');


         // Check if choice is within range

         if (atoi(choice) > 0 && atoi(choice) < pEci->serverCount()+1) {
            strcpy(chosenServer,pEci->serverName(atoi(choice)));
            loop=0;
         }
		 else cout << "ERROR: " << "\"" << choice << "\" is not an a valid choice." << endl << endl;

      }while (loop==1);


      // Open a connection with the chosen server.

      CclConn server(chosenServer, UserID, PassWd);
      CclFlow sflow(Ccl::sync);


      // Start a link to the server with the given transaction as input.

      cout << "Calling CICS program " << progid << endl;
      commarea.setDataLength(ECI_DATA_LENGTH);
      server.link(sflow, progid, &commarea);


      // Check the Execution code and report any failure.

      if (pEci->exCode() != 0) {
         cout << "Call to CICS failed" << endl;
         cout << "Return Code: " << pEci->exCode() << " : " << pEci->exCodeText() << endl;
      } else {
         cout << "Program " << progid << " succeeded." << endl;


         // Cast reply to a char.

         reply = (char*) commarea.dataArea(0);
         cout << "Reply from CICS: " << reply << endl;
      }

   } catch ( CclException& exception ) {

      // Output exception diagnostics

      cout << "CclClass exception:" << exception.diagnose() << endl;
      return 1;
   }

   return 0;
}
