/* File Name(s)  : epib1.cpp

   Product       : CICS Transaction Gateway

   Description   : This sample allows the user to run a transaction such as
                   CESN on a CICS server and outputs the screen contents to
                   the console a field at a time.

 Licensed Materials - Property of IBM  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1999, 2014 All Rights Reserved.  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 2 

                   The following code is sample code created by IBM
                   Corporation.  This sample code is not part of any standard
                   IBM product and is provided to you solely for the purpose
                   of assisting you in the development of your applications.
                   The code is provided 'AS IS', without warranty or
                   condition of any kind.  IBM shall not be liable for any
                   damages arising out of your use of the sample code, even
                   if IBM has been advised of the possibility of such damages. */


// Include files

#include <string.h>
#include <stdlib.h>

#if defined(CICS_W32) || defined(CICS_LNX)
#include <iostream>
using namespace std;
#else
#include <iostream.h>
#endif

#include <cicsepi.hpp>

/* Size of buffer used to read from the console */
#define INPUT_BUFFER_LEN   80
/* Maximum server name length */
#define MAX_SERVER_LEN     8
/* Maximum transid length */
#define MAX_TRANSID_LEN    4

#define copyright_notice copyright4cppepib1
char version4cppepib1[] =
"\nCICS Transaction Gateway"
" Version 9.2.0.3"
" Build Level c920-20180312-1735" ;

char copyright_notice[] =
"\n"
"Licensed Materials - Property of IBM. "
"\n"
"5724-I81,5725-B65 "
"\n"
"(c) Copyright IBM Corp. 1999, 2014 All Rights Reserved. "
"\n"
"US Government Users Restricted Rights - Use, duplication or "
"disclosure restricted by GSA ADP Schedule Contract with "
"IBM Corp."
"\n";


// Main entrypoint

int main(void){
   int i, loop=1;
   int serverSelected = 0;
   char input[INPUT_BUFFER_LEN];
   CclEPI* pepi = NULL;
   CclTerminal* pterm = NULL;
   Ccl::Bool connected = Ccl::no;

   char chosenServer[MAX_SERVER_LEN + 1];
   char transid[MAX_TRANSID_LEN + 1];

   cout << "CICS Transaction Gateway Basic C++ EPI Sample" << endl << endl;


   //Initialise the EPI.

   try {
      pepi = new CclEPI();
   }
   catch (CclException& exception) {


      // Handle the EPI object creation failure.

      cout << "Unable to create an EPI object." << endl;
      cout << exception.diagnose() << endl;
      cout << "Program cannot continue." << endl;
      return 1;
   }


   // Get a server from the user

   do {
      cout << "\nSelect a server or Q to end\n" << endl;
      for (int i = 1; i <= pepi->serverCount(); i++  ) {
         cout << "    " << i << ". " << pepi->serverName(i) << " - "
              << pepi->serverDesc(i) << endl;
      }

      cout << endl << "Choose server to connect to:";
      cout.flush();
      if (cin.peek()=='\n') cin.ignore(1,'\n');
      cin.getline(input, sizeof(input), '\n');

      if (input[0] == 'q' || input[0] == 'Q') {
         delete pepi;
         return 0;
      }
      serverSelected = (unsigned short)atoi(input);
   } while (serverSelected < 1 || serverSelected > pepi->serverCount());
   strcpy(chosenServer,pepi->serverName(serverSelected));

   try {


      //Create a Terminal object on the chosen server.

      cout << "Creating Terminal on server " << chosenServer << endl;
      pterm = new CclTerminal(chosenServer);
      connected = Ccl::yes;


      //Get a transaction ID to send to CICS.

      cout << "Enter the transaction you wish to start:";
      cout.flush();
      if (cin.peek()=='\n') cin.ignore(1,'\n');
      cin.getline(transid, sizeof(transid), '\n');


      //send up transid using a synchronous session

      CclSession session(Ccl::sync);
      pterm->send(&session, transid);


      //Create a screen object and print out the reply from CICS field by field

      CclScreen *screen = pterm->screen();
      for (i=1; i <= screen->fieldCount(); i++ ){
         CclField* field = screen->field(i);
         if ( field->textLength() > 0 ){
            cout << "  Field " << i << ": " << field->text() << endl;
         }
         else {
            cout << "  Field " << i << ": " << endl;
         }
      }


      // Return PF3 to CICS just in case

      screen->setAID( CclScreen::PF3 );
      pterm->send( &session );
   }
   catch (CclException& exception) {
       cout << "Exception occurred: " << exception.diagnose() << endl;
   }


   // Disconnect the terminal.

   if (connected) {
      cout << "Disconnecting terminal" << endl;
      pterm->disconnect();
   }


   // Delete the objects if they were created.

   if (pterm) delete pterm;
   if (pepi)  delete pepi;
   return 0;
}
