      **********************************************************
      *
      * Module Name        EC04.CCP
      *
      * Descriptive Name   CICS server sample program
      *
      * Licensed Materials - Property of IBM  
      *  
      * 5724-I81,5725-B65,5655-Y20 
      *  
      * (C) Copyright IBM Corp. 2015, 2016 All Rights Reserved.  
      *  
      * US Government Users Restricted Rights - Use, duplication or  
      * disclosure restricted by GSA ADP Schedule Contract with  
      * IBM Corp.  
      *
      * Status: Version 9 Release 2 
      *
      * EC04 is a CICS server Cobol sample program which returns
      * to the client application the following data in a COMMAREA:
      *   time
      *   date
      *   applid of CICS region
      *   version of CICS or TXSeries
      *   userid the transaction ran under
      *   mirror used
      *   transid used
      *   protocol (e.g. IPIC, SNA, EXCI)
      *
      * TXSeries users will need to comment out some statements where
      * the SPI is not supported. Search for TXSeries in this source.
      *
      * EC04 can be called using an External Call Interface (ECI) request
      * using, for example, Java sample EciB2, or a DPL request between
      * two CICS servers.
      *
      * It expects a COMMAREA length of 122 bytes.
      * Abend CLEN caught if COMMAREA not equal to 122.
      *
      * Example output of COMMAREA:
      * date=01/05/15,time=14:22:18,applid=CICSAPPL,version=CICS TS  5.2,
      * userid=CICSUSER,mirror=CPMI,transid=CPMI,protocol=IPIC
      *
      * The following code is sample code created by IBM Corporation.
      * This sample code is not part of any standard IBM product and
      * is provided to you solely for the purpose of assisting you in
      * the development of your applications.  The code is provided
      * 'as is', without warranty or condition of any kind.  IBM shall
      * not be liable for any damages arising out of your use of the
      * sample code, even if IBM has been advised of the possibility
      * of such damages.
      *
      ***************************************************************
      *
       Identification Division.
      *
       Program-Id. EC04.
      *
       Environment Division.
      *
       Data Division.
      *
       Working-Storage Section.
      *
       01 ab-code             pic X(4)  value 'CLEN'.
       01 ws-cics-resp        pic s9(8) binary.
       01 ws-raw-time         pic s9(15) comp-3.
       01 ws-task             pic s9(7) comp-3.
       01 ws-uow              pic x(16) value spaces.
       01 ws-uow-inq          pic x(16) value spaces.
       01 ws-cics-platform    pic x(1) value space.
       01 ws-cics-ts-level    pic x(6) value spaces.
       01 ws-txseries-level   pic x(4) value spaces.
       01 ws-cics-vse-level   pic x(4) value spaces.
       01 ws-cics-protocol    pic s9(8) comp.
       01 ws-uowlink          pic s9(8) comp.
      *
      * Working Storage for COMMAREA to be returned to client
      *
       01 ca-commarea.
          03 ca-date-filler            pic x(5) value "date=".
          03 ca-date-string            pic x(8) value spaces.
          03 ca-time-filler            pic x(6) value ",time=".
          03 ca-time-string            pic x(8) value spaces.
          03 ca-cics-applid-filler     pic x(8) value ",applid=".
          03 ca-cics-applid            pic x(8) value spaces.
          03 ca-cics-version.
             05 ca-cics-version-filler pic x(9) value ",version=".
             05 ca-cics-product        pic x(8) value spaces.
             05 ca-cics-level          pic x(4) value spaces.
          03 ca-cics-userid-filler     pic x(8) value ",userid=".
          03 ca-cics-userid            pic x(8) value spaces.
          03 ca-mirror-filler          pic x(8) value ",mirror=".
          03 ca-mirror                 pic x(4) value spaces.
          03 ca-transid-filler         pic x(9) value ",transid=".
          03 ca-transid                pic x(4) value spaces.
          03 ca-cics-protocol-filler   pic x(10) value ",protocol=".
          03 ca-cics-protocol          pic x(7) value spaces.
      *
       Linkage Section.
      *
       01 dfhcommarea.
          03 lk-rest-commarea pic X(122).
      *
       Procedure Division.
      *
      * If commarea length not 122 return abend
           if eibcalen not = 122
              perform Force-Abend
           else
      * Gather information about CICS
              perform Get-TimeStamp
              perform Get-CICS-Applid
              perform Get-CICS-Platform
              perform Get-CICS-Userid
              perform Get-CICS-Mirror
              perform Get-CICS-Transid
              perform Get-CICS-Protocol
              perform Done
           end-if.
      *
       Force-Abend.
      * Abend the program
      * NOTE - The NODUMP option is not supported on all
      *        CICS platforms
      *
           EXEC CICS ABEND
                ABCODE(ab-code)
                NODUMP
           END-EXEC.
      *
       Done.
      *    Copy the populated COMMAREA into the Linkage Section
           move ca-commarea to dfhcommarea.
      *
           EXEC CICS RETURN
           END-EXEC.
      *
       Get-TimeStamp.
      *  Get the current time and date
           EXEC CICS ASKTIME ABSTIME(ws-raw-time)
              RESP(ws-cics-resp)
           END-EXEC
           perform Check-CICS-Response

      *  Format date and time
           EXEC CICS FORMATTIME ABSTIME(ws-raw-time)
                            DDMMYY(ca-date-string) DATESEP('/')
                            TIME(ca-time-string) TIMESEP(':')
                            RESP(ws-cics-resp)
                            END-EXEC
           perform Check-CICS-Response
           .
      *
       Get-CICS-Applid.
      *  Get CICS Applid
           EXEC CICS ASSIGN APPLID(ca-cics-applid)
              RESP(ws-cics-resp)
           END-EXEC
           perform Check-CICS-Response
           .
      *
       Get-CICS-Platform.
      *  Get CICS Platform - X for CICS TS, A/H/L/N/S for TXSeries
      *                    - F for CICS VSE
           EXEC CICS INQUIRE SYSTEM
              CICSSYS(ws-cics-platform)
              RESP(ws-cics-resp)
           END-EXEC
           perform Check-CICS-Response
      *
           evaluate ws-cics-platform
      *  For TXSeries, comment out the following When statement
      *  as CICSTSLEVEL not supported.
      *  When CICS TS ...
              when 'X'
                 move 'CICS TS ' to ca-cics-product
                 EXEC CICS INQUIRE SYSTEM
                    CICSTSLEVEL(ws-cics-ts-level)
                    RESP(ws-cics-resp)
                 END-EXEC
                 perform Check-CICS-Response
      *  Response is 6 chars, vvmmrr, so lose release rr
                 move ws-cics-ts-level(1:4) to ca-cics-level
      *  Smarten up output
              if ca-cics-level(1:1) = '0'
                 move space to ca-cics-level(1:1)
              end-if
              if ca-cics-level(3:1) = '0'
                 move '.' to ca-cics-level(3:1)
              end-if
      *  When TXSeries ...
              when 'A'
              when 'H'
              when 'L'
              when 'N'
              when 'S'
                 move 'TXSeries' to ca-cics-product
                 EXEC CICS INQUIRE SYSTEM
                    RELEASE(ws-txseries-level)
                    RESP(ws-cics-resp)
                 END-EXEC
                 perform Check-CICS-Response
      *  Response is 4 chars, vvmm
      *  Smarten up output
              if ws-txseries-level(1:1) = '0'
                 move space to ws-txseries-level(1:1)
              end-if
              move ws-txseries-level to ca-cics-level
      *  When CICS VSE ...
              when 'F'
                 move 'CICS VSE' to ca-cics-product
                 EXEC CICS INQUIRE SYSTEM
                    RELEASE(ws-cics-vse-level)
                    RESP(ws-cics-resp)
                 END-EXEC
                 perform Check-CICS-Response
      *  Response is 4 chars, vvmm
                 move ws-cics-vse-level to ca-cics-level
      *  Unknown platform
              when other
                 move 'Unknown' to ca-cics-product
           end-evaluate
           .
      *
       Get-CICS-Userid.
      *  Get userid that mirror transaction is using
           EXEC CICS ASSIGN USERID(ca-cics-userid)
              RESP(ws-cics-resp)
           END-EXEC
           perform Check-CICS-Response
           .
      *
       Get-CICS-Mirror.
      *  Get CICS mirror associated with the task
           EXEC CICS INQUIRE ASSOCIATION(EIBTASKN)
              TRANSACTION(ca-mirror)
              RESP(ws-cics-resp)
           END-EXEC
           perform Check-CICS-Response
      *  TXSeries does not support ASSOCIATION.
      *  Comment out the above EXEC CICS call and
      *  uncomment the following line:
      *  move EIBTRNID to ca-mirror
           .
      *
       Get-CICS-Transid.
      *  Get CICS transid from eibtrnid
      *  This is useful as an alternative mirror may
      *  have been specified by the client
           move EIBTRNID to ca-transid
           .
      *
       Get-CICS-Protocol.
      *  Not supported on TXSeries
      *  Set to "Unknown" when running on TXSeries
           move 'Unknown' to ca-cics-protocol
      *  For TXSeries users, comment out the
      *  rest of this procedure.
      *
      *  For CICS TS and CICS VSE
      *  establish if TCPIP, IPIC, SNA or EXCI
           move EIBTASKN to ws-task
           EXEC CICS INQUIRE TASK(ws-task)
              UOW(ws-uow)
              RESP(ws-cics-resp)
           END-EXEC
           perform Check-CICS-Response
           EXEC CICS INQUIRE UOWLINK START
              RESP(ws-cics-resp)
           END-EXEC
           perform Check-CICS-Response
           perform until ws-cics-resp = DFHRESP(END)
                   OR ws-uow-inq(1:8) = ws-uow(1:8)
              EXEC CICS INQUIRE UOWLINK(ws-uowlink) NEXT
                 PROTOCOL(ws-cics-protocol)
                 UOW(ws-uow-inq)
                 RESP(ws-cics-resp)
              END-EXEC
              if ws-cics-resp = DFHRESP(END)
                 move 'Unknown' to ca-cics-protocol
              end-if
           end-perform
           EXEC CICS INQUIRE UOWLINK END
                 RESP(ws-cics-resp)
           END-EXEC
      *
           evaluate ws-cics-protocol
              when 1089
                 move 'IPIC ' to ca-cics-protocol
              when 121
                 move 'EXCI ' to ca-cics-protocol
              when 124
                 move 'SNA  ' to ca-cics-protocol
      *    TCPIP does not have UOWLINK so need to assume TCPIP
              when other
                 move 'TCPIP' to ca-cics-protocol
           end-evaluate
           .
      *
       Check-CICS-Response.
           if ws-cics-resp NOT = dfhresp(NORMAL)
              move 'ERR ' to ab-code
              perform Force-Abend
           end-if
           .
