/*
   MODULE NAME        CICS_EPI.H

   DESCRIPTIVE NAME   CICS External Presentation Interface

 Licensed Materials - Property of IBM  
  
 "Restricted Materials of IBM"  
  
 5724-I81,5725-B65 
  
 (C) Copyright IBM Corp. 1994, 2014  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
   
   Status: Version 9 Release 2 

   NOTES :-

   This header file is provided with the CICS Transaction Gateway. CICS TG
   is available for a number of different operating environments and to ensure
   the correct sections of this header are included, one of the following
   constants must be defined prior to inclusion:

     CICS_W32  -  if building a Windows application
     CICS_AIX  -  if building an AIX application
     CICS_SOL  -  if building a Solaris application
     CICS_HPUX -  if building an HPUX application
     CICS_LNX  -  if building a Linux application
     CICS_HPIT and CICS_HPUX -
                  if building an HPUX application on Itanium 
*/

#ifndef _CICS_EPI_H
#define _CICS_EPI_H

#include <cicstype.h>

#ifdef __cplusplus
  extern "C" {
#endif


#if defined(CICS_AIX)
#pragma options align=packed
#elif defined(CICS_HPUX)
#pragma pack 1
#else
#pragma pack(1)
#endif

/*
   Constants
*/

/* Lengths of fields   */
#define CICS_EPI_SYSTEM_MAX            8
#define CICS_EPI_DESCRIPTION_MAX      60
#define CICS_EPI_NETNAME_MAX           8
#define CICS_EPI_TRANSID_MAX           4
#define CICS_EPI_DEVTYPE_MAX          16
#define CICS_EPI_ERROR_MAX            60
#define CICS_EPI_PASSWORD_MAX         10
#define CICS_EPI_USERID_MAX           10
#define CICS_EPI_MAPNAME_MAX           7
#define CICS_EPI_MAPSETNAME_MAX        8
#define CICS_EPI_TERMID_MAX            4

/* Constants relating to TermIndex */
#define CICS_EPI_TERM_INDEX_NONE  0xFFFF
#define CICS_EPI_MIN_TERMINDEX         0

/* Current Version Number */
#define CICS_EPI_VERSION_200  0x00020000

/* Return values for EPI calls */
#define CICS_EPI_NORMAL                  0
#define CICS_EPI_ERR_NOT_INIT            1
#define CICS_EPI_ERR_BAD_INDEX           2
#define CICS_EPI_ERR_FAILED              3
#define CICS_EPI_ERR_VERSION             4
#define CICS_EPI_ERR_IS_INIT             5
#define CICS_EPI_ERR_NO_SYSTEMS          6
#define CICS_EPI_ERR_MORE_SYSTEMS        7
#define CICS_EPI_ERR_MAX_TERMS           8
#define CICS_EPI_ERR_SYSTEM              9
#define CICS_EPI_ERR_TRAN_ACTIVE        10
#define CICS_EPI_ERR_NO_DATA            11
#define CICS_EPI_ERR_TTI_ACTIVE         12
#define CICS_EPI_ERR_ATI_ACTIVE         13
#define CICS_EPI_ERR_NO_CONVERSE        14
#define CICS_EPI_ERR_ATI_STATE          15
#define CICS_EPI_ERR_NO_EVENT           16
#define CICS_EPI_ERR_MORE_EVENTS        17
#define CICS_EPI_ERR_MORE_DATA          18
#define CICS_EPI_ERR_WAIT               19
#define CICS_EPI_ERR_SERVER_DOWN        20
#define CICS_EPI_ERR_SENSE_CODE         21
#define CICS_EPI_ERR_SECURITY           22
#define CICS_EPI_ERR_IN_CALLBACK        23
#define CICS_EPI_ERR_NULL_PARM          24
#define CICS_EPI_ERR_ADDTYPE_INVALID    25
#define CICS_EPI_ERR_SIGNONCAP_INVALID  26
#define CICS_EPI_ERR_RESPONSE_TIMEOUT   27
#define CICS_EPI_ERR_SIGNON_NOT_POSS    28
#define CICS_EPI_ERR_TERMID_INVALID     29
#define CICS_EPI_ERR_MODELID_INVALID    30
#define CICS_EPI_ERR_NOT_3270_DEVICE    31
#define CICS_EPI_ERR_ALREADY_INSTALLED  32
#define CICS_EPI_ERR_USERID_INVALID     33
#define CICS_EPI_ERR_PASSWORD_INVALID   34
#define CICS_EPI_ERR_NULL_USERID        35
#define CICS_EPI_ERR_NULL_PASSWORD      36
#define CICS_EPI_ERR_CCSID_INVALID      37
#define CICS_EPI_ERR_SERVER_BUSY        38
#define CICS_EPI_ERR_ABENDED            39
#define CICS_EPI_ERR_RESOURCE_SHORTAGE  40
#define CICS_EPI_ERR_NO_CICS            41
#define CICS_EPI_ERR_MAX_SYSTEMS        42
#define CICS_EPI_ERR_MAX_SESSIONS       43

/*
   Data Types and Values
*/
typedef cics_ushort_t CICS_EpiAdd_t;
#define CICS_EPI_ADD_SYNC        0
#define CICS_EPI_ADD_ASYNC       1

typedef cics_ushort_t CICS_EpiSignon_t;
#define CICS_EPI_SIGNON_CAPABLE      0
#define CICS_EPI_SIGNON_INCAPABLE    1
#define CICS_EPI_SIGNON_UNKNOWN      2

typedef cics_ushort_t CICS_EpiEnd_t;

/* End Reasons for CICS_EPI_EVENT_END_TERM event */
#define CICS_EPI_END_SIGNOFF     1
#define CICS_EPI_END_SHUTDOWN    2
#define CICS_EPI_END_OUTSERVICE  3
#define CICS_EPI_END_UNKNOWN     4
#define CICS_EPI_END_FAILED      5

/* End Reasons for CICS_EPI_EVENT_END_TRAN event */
#define CICS_EPI_TRAN_NO_ERROR        0
#define CICS_EPI_TRAN_NOT_STARTED    10
#define CICS_EPI_TRAN_STATE_UNKNOWN  11
#define CICS_EPI_READTIMEOUT_EXPIRED 15

typedef cics_ushort_t CICS_EpiEvent_t;
#define CICS_EPI_EVENT_SEND      1
#define CICS_EPI_EVENT_CONVERSE  2
#define CICS_EPI_EVENT_END_TRAN  3
#define CICS_EPI_EVENT_START_ATI 4
#define CICS_EPI_EVENT_END_TERM  5
#define CICS_EPI_EVENT_ADD_TERM  6

typedef cics_ushort_t CICS_EpiATIState_t;
#define CICS_EPI_ATI_ON          1
#define CICS_EPI_ATI_HOLD        2
#define CICS_EPI_ATI_QUERY       3

typedef cics_ushort_t CICS_EpiWait_t;
#define CICS_EPI_WAIT            1
#define CICS_EPI_NOWAIT          2

typedef struct
{
    cics_char_t           SystemName[CICS_EPI_SYSTEM_MAX+1];
    cics_char_t           Description[CICS_EPI_DESCRIPTION_MAX+1];
} CICS_EpiSystem_t;

typedef struct
{
    CICS_EpiAdd_t         EpiAddType;
    cics_ushort_t         InstallTimeOut;
    cics_ushort_t         ReadTimeOut;
    CICS_EpiSignon_t      SignonCapability;
    cics_ushort_t         CCSId;
    cics_char_t           UserId[CICS_EPI_USERID_MAX+1];
    cics_char_t           Password[CICS_EPI_PASSWORD_MAX+1];
} CICS_EpiAttributes_t;

typedef struct
{
    cics_char_t           NetName[CICS_EPI_NETNAME_MAX+1];
    cics_sshort_t         NumLines;
    cics_sshort_t         NumColumns;
    cics_ushort_t         MaxData;
    cics_sshort_t         ErrLastLine;
    cics_sshort_t         ErrIntensify;
    cics_sshort_t         ErrColor;
    cics_sshort_t         ErrHilight;
    cics_sshort_t         Hilight;
    cics_sshort_t         Color;
    cics_sshort_t         Reserved2;
    cics_char_t           System[CICS_EPI_SYSTEM_MAX+1];
    cics_char_t           TermId[CICS_EPI_TERMID_MAX+1];
    CICS_EpiSignon_t      SignonCapability;
} CICS_EpiDetails_t;

typedef struct
{
    cics_ushort_t         TermIndex;
    CICS_EpiEvent_t       Event;
    CICS_EpiEnd_t         EndReason;
    cics_char_t           TransId[CICS_EPI_TRANSID_MAX+1];
    cics_char_t           Reserved1[5];
    cics_ubyte_t CICSPTR *Data;
    cics_ushort_t         Size;
    cics_sshort_t         EndReturnCode;
    cics_char_t           MapName[CICS_EPI_MAPNAME_MAX+1];
    cics_char_t           MapSetName[CICS_EPI_MAPSETNAME_MAX+1];
} CICS_EpiEventData_t;


/* EPI callback notification function definition. */
typedef void (CICSEXIT CICS_EpiCallback_t)(cics_ushort_t TermIndex);
typedef CICS_EpiCallback_t CICSPTR *CICS_EpiNotify_t;


/*
   Function prototypes for EPI.
*/

cics_sshort_t CICSCALL CICS_EpiInitialize(cics_ulong_t  Version);

cics_sshort_t CICSCALL CICS_EpiTerminate(void);

cics_sshort_t CICSCALL CICS_EpiListSystems(cics_char_t      CICSPTR *NameSpace,
                                           cics_ushort_t    CICSPTR *Systems,
                                           CICS_EpiSystem_t CICSPTR *List);

cics_sshort_t CICSCALL
              CICS_EpiAddTerminal(cics_char_t       CICSPTR *NameSpace,
                                  cics_char_t       CICSPTR *System,
                                  cics_char_t       CICSPTR *NetName,
                                  cics_char_t       CICSPTR *DevType,
                                  CICS_EpiNotify_t           NotifyFn,
                                  CICS_EpiDetails_t CICSPTR *Details,
                                  cics_ushort_t     CICSPTR *TermIndex);

cics_sshort_t CICSCALL
              CICS_EpiAddExTerminal(cics_char_t          CICSPTR *System,
                                    cics_char_t          CICSPTR *NetName,
                                    cics_char_t          CICSPTR *DevType,
                                    CICS_EpiNotify_t              NotifyFn,
                                    CICS_EpiDetails_t    CICSPTR *Details,
                                    cics_ushort_t        CICSPTR *TermIndex,
                                    CICS_EpiAttributes_t CICSPTR *Attributes);

cics_sshort_t CICSCALL CICS_EpiSetSecurity(cics_ushort_t          TermIndex,
                                           cics_char_t   CICSPTR *UserId,
                                           cics_char_t   CICSPTR *Password);

cics_sshort_t CICSCALL CICS_EpiInquireSystem(cics_ushort_t        TermIndex,
                                             cics_char_t CICSPTR *System);

cics_sshort_t CICSCALL CICS_EpiDelTerminal(cics_ushort_t TermIndex);

cics_sshort_t CICSCALL CICS_EpiPurgeTerminal(cics_ushort_t TermIndex);

cics_sshort_t CICSCALL CICS_EpiStartTran(cics_ushort_t         TermIndex,
                                         cics_char_t  CICSPTR *TransId,
                                         cics_ubyte_t CICSPTR *Data,
                                         cics_ushort_t         Size);

cics_sshort_t CICSCALL CICS_EpiReply(cics_ushort_t         TermIndex,
                                     cics_ubyte_t CICSPTR *Data,
                                     cics_ushort_t         Size);

cics_sshort_t CICSCALL CICS_EpiATIState(cics_ushort_t               TermIndex,
                                        CICS_EpiATIState_t CICSPTR *ATIState);

cics_sshort_t CICSCALL CICS_EpiGetEvent(cics_ushort_t                TermIndex,
                                        CICS_EpiWait_t               Wait,
                                        CICS_EpiEventData_t CICSPTR *Event);


#if defined(CICS_AIX)
#pragma options align=reset
#elif defined(CICS_HPUX) && defined(__cplusplus)
#pragma pack
#elif defined(CICS_HPUX) && defined(CICS_HPIT)
#pragma pack
#elif defined(CICS_HPUX)
#pragma HP_ALIGN HPUX_NATURAL
#else
#pragma pack()
#endif

#ifdef __cplusplus
  }
#endif

#endif /* _CICS_EPI_H */
