??=ifdef __COMPILER_VER__
  ??=pragma filetag("IBM-1047")
??=endif
/*
   File Name(s)  : ctgsmf.h

   Product       : CICS Transaction Gateway

   Description   : This header file describes the format of the
                   records that the CICS TG writes to SMF.

 Licensed Materials - Property of IBM  
  
 "Restricted Materials of IBM"  
  
 5724-I81,5725-B65,5655-Y20 
  
 (C) Copyright IBM Corp. 2007, 2014  
  
 US Government Users Restricted Rights - Use, duplication or  
 disclosure restricted by GSA ADP Schedule Contract with  
 IBM Corp.  
  
  Status: Version 9 Release 2 

  NOTES :-
  See product documentation for further
  information on the record format.

*/

#if !defined(_SMFHEADERS)
#define _SMFHEADERS

#pragma pack(packed)

/**********************************************************************/
/* SMF record structure                                               */
/**********************************************************************/
/* The SMF record contains data in the following order                */
/* SMF_HEADER                                                         */
/* SMF_GENERIC_PRODUCT_SECTION                                        */
/* SMF_CTG_PRODUCT_SECTION                                            */
/* CTG_RECORD_HEADER + statistics data section                        */
/* There may be any number of CTG_RECORD_HEADERS + data sections      */
/* See the product documentation for more details.                    */
/**********************************************************************/
/**********************************************************************/
/* SMF_HEADER                                                         */
/* The following fields are recorded for all applications that        */
/* write to SMF.                                                      */
/**********************************************************************/
typedef struct {
   unsigned short smf111_len;            /* Length of record          */
#define SMFLEN_MAX 30720                 /* Maximum size allowed for  */
                                         /* a SMF record.             */
   unsigned short smf111_seg;            /* Segment descriptor        */
   char smf111_flg;                      /* O/S indicator             */
     #define SMFFLG_SUBTYPES_ON 0xC0
   char smf111_rty;                      /* Record type               */
     #define SMFRTRY_CTG_RECORD_TYPE 111 /* The record type reserved  */
                                         /* for the CICS TG.          */
   unsigned int smf111_time;             /* Time moved to SMF         */
   unsigned int smf111_sdte;             /* Date moved to SMF         */
   char smf111_sid[4];                   /* System identification     */
   char smf111_ssi[4];                   /* Subsystem identification  */
     #define SMFSSI_CTG_IDENTIFIER "CTGZ"
   unsigned short smf111_sty;            /* Record subtype            */
     #define SMFSTY_STATS_SUBTYPE 0x0000 /* The CICS TG currently     */
                                         /* only uses 1 subtype       */
   unsigned short smf111_trn;            /* Number of triplets        */
     #define SMFTRN_DEFAULT 2
   char reserved[2];              /* Unused. Aligns subsequent fields.*/
   unsigned int smf111_aps;             /* Offset to product section  */
   unsigned short smf111_lps;           /* Length of product section  */
   unsigned short smf111_nps;           /* Number of product sections */
     #define SMFNPS_DEFAULT 1
   unsigned int smf111_ass;             /* Offset to data section     */
   unsigned short smf111_asl;           /* Length of data section     */
   unsigned short smf111_asn;           /* Number of data sections    */
     #define SMFASN_DEFAULT 1
} SMF_HEADER;
/**********************************************************************/
/* SMF_GENERIC_PRODUCT_SECTION                                        */
/**********************************************************************/
typedef struct {
   unsigned short smf111_vrm;           /* Record version (0xVRM)     */
     #define CURRENT_SMF111_VRM 0x920
   char smf111_prn[8];                  /* Generic product name       */
    #define SMFPRNT_CTG_GENERIC_PRODUCT_NAME "CICSTGGW"
   char smf111_spn[8];                  /* Gateway daemon APPLID      */
   unsigned short smf111_mfl;           /* Record maintenance level   */
    #define CURRENT_SMF111_MFL 0x3
   char reserved[2];              /* Unused. Aligns subsequent fields.*/
} SMF_GENERIC_PRODUCT_SECTION;

/**********************************************************************/
/* SMF_CTG_PRODUCT_SECTION                                            */
/**********************************************************************/
/* The following fields are specific to the CICS TG                   */
/* See the product documentation for further details on the format    */
/* of the fields.                                                     */
/**********************************************************************/
typedef struct {
   char ctg_stattype;         /*The statistics event being recorded.  */
       #define CTGSTATTYPE_INTERVAL 0x00
       #define CTGSTATTYPE_EOD 0x01
       #define CTGSTATTYPE_SHUTDOWN 0x02
   char reserved1;            /* Unused. Aligns subsequent fields.    */
   char ctg_coldate[4];       /* Collection date (0CYYDDD+)           */
                              /* Local time                           */
                              /* Binary Coded Decimal                 */
                              /* YY = year                            */
                              /* DDD = day of year                    */
                              /* C is the number of centuries since   */
                              /* 1900.                                */
   char ctg_coltime[4];       /* Collection time (00HHMMSS)           */
                              /* Local time                           */
                              /* Binary Coded Decimal                 */
                              /* HH = hour                            */
                              /* MM = minute                          */
                              /* SS = seconds                         */
                              /* 23:59.59 is the last second          */
                              /* of the current day.                  */
   int ctg_locoffset;         /* Offset from GMT in seconds           */
                              /* If local time is behind GMT          */
                              /* this is a negative value.            */
   int ctg_lstreset;          /* Seconds since last reset, or         */
                              /* initialization                       */
   int ctg_interval;          /* Time in seconds to next              */
                              /* interval.                            */
   int ctg_intvcount;         /* Sequence number of interval          */
                              /* Starts at 1 when the gateway         */
                              /* daemon is initialized.               */
   char ctg_startdate[4];     /* CICS TG start date (0CYYDDD+)        */
                              /* Local time                           */
                              /* Binary Coded Decimal                 */
                              /* YY = year                            */
                              /* DDD = day of year                    */
                              /* C is the number of centuries since   */
                              /* 1900.                                */
   char ctg_starttime[4];     /* CICS TG start time (00HHMMSS)        */
                              /* Local time                           */
                              /* Binary Coded Decimal                 */
                              /* HH = hour                            */
                              /* MM = minute                          */
                              /* SS = seconds                         */
                              /* 23:59.59 is the last second          */
                              /* of the current day.                  */
   char reserved2[6];         /* Make the structure a multiple of     */
                              /* 8 bytes.                             */
   } SMF_CTG_PRODUCT_SECTION;

/**********************************************************************/
/* CICS TG record header                                              */
/**********************************************************************/
typedef struct {
   short ctg_datalen;   /* Length of this header + data that follows  */
   short ctg_recordid;  /* Identifies the resource group the data     */
                        /* represents.                                */
      #define CM_RECORDID 0
      #define CS_RECORDID 1
      #define CSX_RECORDID 2
      #define GD_RECORDID 3
      #define PH_RECORDID 4
      #define WT_RECORDID 5
      #define SE_RECORDID 6
      #define CSX_IPIC_RECORDID 7
      #define WS_RECORDID 8
      #define WSX_RECORDID 9
   char  ctg_recver;    /* Version number of the CTG record           */
   char reserved[3];    /* Make the structure a multiple of 8 bytes   */
} CTG_RECORD_HEADER;

/**********************************************************************/
/* CTG_RECORD_HEADER + statistics data section                        */
/**********************************************************************/
/* NOTES on naming convention for field names                         */
/* ctg<RESOURCEGROUP><c/s/l/i><STATNAME>                              */
/* where c = current, s = static, l = lifetime and i= interval.       */
/* The product documentation contains more details on the record      */
/* structure and the definitions of the statistics that the fields    */
/* represent.                                                         */
/* NOTES on structure lengths                                         */
/* All structures are required to be a multiple of 8 bytes. If the    */
/* sum of lengths of all fields is not a multiple of 8, a reserved    */
/* field is inserted at the end of the structure to pad it out.       */
/**********************************************************************/

/* Data section of PH resource group statistics record.               */
typedef struct {
    int ctg_ph_sportssl;        /* SSL protocol handler port number   */
    int ctg_ph_sporttcp;        /* TCP protocol handler port number   */
    char ctg_ph_sbindtcp[256];  /* TCP protocol handler bind address  */
    char ctg_ph_sbindssl[256];  /* SSL protocol handler bind address  */
    int ctg_ph_sporthttp;       /* HTTP protocol handler port number  */
    int ctg_ph_sporthttps;      /* HTTPS protocol handler port number */
    char ctg_ph_sbindhttp[256]; /* HTTP protocol handler bind address */
    char ctg_ph_sbindhttps[256];/* HTTPS protocol handler bind address*/
} PH_DATA;
#define PH_DATA_VERSION 0x00  /* ctg_recver for this structure.       */

/* Data section of CSx for EXCI resource group statistics record.     */
typedef struct {
    char ctg_csx_sapplid[8]; /* APPLID of CICS server                 */
    int  ctg_csx_calloc;     /* Number of EXCI pipes allocated        */
    int  ctg_csx_lallocfail; /* Number of EXCI pipe allocate failures */
    int  ctg_csx_lallreq;    /* Number of requests executed           */
    int  ctg_csx_iallocfail; /* Number of EXCI pipe allocate failures */
    int  ctg_csx_iallreq;    /* Number of requests executed           */
    int  ctg_csx_iavresp;    /* Average CICS response time (ms)       */
    int  ctg_csx_lavresp;    /* Average CICS response time (ms)       */
    int  ctg_csx_lcommsfail; /* Number of CICS communication failures */
    long long ctg_csx_ireqdata;  /* Amount of CICS request data       */
    long long ctg_csx_lreqdata;  /* Amount of CICS request data       */
    long long ctg_csx_irespdata; /* Amount of CICS response data      */
    long long ctg_csx_lrespdata; /* Amount of CICS response data      */
    char ctg_csx_sprotocol[8];   /* CICS server protocol              */
    #define CSX_PROTOCOL_EXCI  "EXCI    "
    int  ctg_csx_icommsfail; /* Number of CICS communication failures */
    int  ctg_csx_cwaiting;   /* Number of requests waiting on a       */
                             /* response                              */
} CSx_EXCI_DATA;
#define CSX_EXCI_DATA_VERSION 0x00 /* ctg_recver for this structure.*/

/* Data section of CSx for IPIC resource group statistics record.     */
typedef struct {
    char ctg_csx_sininame[8];/* Server name as configured in ini file */
    long long ctg_csx_ireqdata; /* Amount of CICS request data        */
    long long ctg_csx_lreqdata; /* Amount of CICS request data        */
    long long ctg_csx_irespdata;/* Amount of CICS response data       */
    long long ctg_csx_lrespdata;/* Amount of CICS response data       */
    int ctg_csx_iallreq;        /* Number of requests processed       */
    int ctg_csx_lallreq;        /* Number of requests processed       */
    int ctg_csx_iconnfail;      /* Number of connection failures      */
    int ctg_csx_lconnfail;      /* Number of connection failures      */
    int ctg_csx_ilostconn;      /* Number of link failures            */
    int ctg_csx_llostconn;      /* Number of link failures            */
    int ctg_csx_iidletimeout;   /* Number of requests timed out       */
    int ctg_csx_lidletimeout;   /* Number of requests timed out       */
    char ctg_csx_sipaddr[104];  /* CICS server TCP/IP address         */
    int ctg_csx_sipport;        /* CICS server TCP/IP port            */
    int ctg_csx_csesscurr;      /* Number of IPIC sessions in use     */
    int ctg_csx_csessmax;       /* Number of negotiated IPIC sessions */
    int ctg_csx_ssessmax;       /* Number of requested IPIC sessions  */
    int ctg_csx_lsessfail;      /* Number of IPIC session failures    */
    int ctg_csx_isessfail;      /* Number of IPIC session failures    */
    char ctg_csx_sprotocol[8];  /* CICS server protocol               */
    #define CSX_PROTOCOL_IPIC  "IPIC    "
    int ctg_csx_lcommsfail;  /* Number of CICS communication failures */
    int ctg_csx_icommsfail;  /* Number of CICS communication failures */
    int ctg_csx_cwaiting; /* Number of requests waiting on a response */
    int ctg_csx_iavresp;        /* Average CICS response time (ms)    */
    int ctg_csx_lavresp;        /* Average CICS response time (ms)    */
    char ctg_csx_capplidq[8];   /* IPIC Network id of CICS server     */
    char ctg_csx_capplid[8];    /* APPLID of CICS server              */
    char ctg_csx_cstatus[16];   /* CICS server status                 */
    char reserved[4];           /* Pads to a multiple of 8 bytes      */
} CSx_IPIC_DATA;
#define CSx_IPIC_DATA_VERSION 0x00 /* ctg_recver for this structure */

/* Data section for CM resource group statistics record               */
typedef struct {
    int  ctg_cm_calloc;    /* Currently allocated connection managers */
    int  ctg_cm_ccurr;     /* Current number of connection managers   */
    int  ctg_cm_cwaiting;  /* Connection managers waiting for a worker*/
    int  ctg_cm_ltimeouts; /* Number of times connecttimeout hit      */
    int  ctg_cm_sinit;     /* Initial number of connection managers   */
    int  ctg_cm_smax;      /* Maximum number of connection managers   */
    int  ctg_cm_itimeouts; /* Number of times connecttimeout hit      */
    int  ctg_cm_iallochi;  /* A high water mark for cm_calloc.        */
    int  ctg_cm_icreated;  /* Number of connection manager threads    */
                           /* created.                                */
    int  ctg_cm_ialloc;    /* Number of times a connection manager    */
                           /* thread was allocated.                   */
    int  ctg_cm_lalloc;    /* Number of times a connection manager    */
                           /* thread was allocated.                   */
    char reserved[4];      /* Pads to a multiple of 8 bytes           */
} CM_DATA;
#define CM_DATA_VERSION 0x00 /* ctg_recver for this structure.        */

/* Data section for CS resource group statistics record               */
typedef struct {
    int  ctg_cs_calloc;     /* Number of EXCI pipes allocated         */
    int  ctg_cs_lallocfail; /* Number of EXCI pipe allocate failures  */
    int  ctg_cs_lallreq;    /* Number of requests executed            */
    int  ctg_cs_lcommsfail; /* Number of CICS communication failures  */
    int  ctg_cs_lcount;     /* Number of CICS regions                 */
    int  ctg_cs_lrealloc;   /* Number of EXCI pipe reallocations      */
    int  ctg_cs_slogonlim;  /* EXCI pipe limit                        */
    char ctg_cs_snetname[8];/* EXCI netname, defined by DFHJVPIPE     */
    int  ctg_cs_iallocfail; /* Number of EXCI allocate failures       */
    int  ctg_cs_iallreq;    /* Number of requests executed            */
    int  ctg_cs_icommsfail; /* Number of CICS communication failures  */
    int  ctg_cs_icount;     /* Number of CICS regions                 */
    int  ctg_cs_irealloc;   /* Number of EXCI pipe reallocations      */
    long long ctg_cs_ireqdata; /* Amount of CICS request data         */
    long long ctg_cs_lreqdata; /* Amount of CICS request data         */
    long long ctg_cs_irespdata;/* Amount of CICS response data        */
    long long ctg_cs_lrespdata;/* Amount of CICS response data        */
    int ctg_cs_scount;      /* Number of defined CICS servers         */
    int ctg_cs_iconnfail;   /* Number of connect failures             */
    int ctg_cs_lconnfail;   /* Number of connect failures             */
    int ctg_cs_ilostconn;   /* Number of lost connections             */
    int ctg_cs_llostconn;   /* Number of lost connections             */
    int ctg_cs_lidletimeout;/* Number of requests timed out           */
    int ctg_cs_csesscurr;   /* Number of IPIC sessions in use         */
    int ctg_cs_csessmax;    /* Number of negotiated IPIC sessions     */
    int ctg_cs_lsessfail;   /* Number of IPIC session failures        */
    int ctg_cs_isessfail;   /* Number of IPIC session failures        */
    int ctg_cs_cwaiting;  /* Number of requests waiting on a response */
    int ctg_cs_iidletimeout;/* Number of requests timed out           */
    int ctg_cs_iavresp;     /* Average CICS response time (ms)        */
    int ctg_cs_lavresp;     /* Average CICS response time (ms)        */
} CS_DATA;
#define CS_DATA_VERSION 0x00 /* ctg_recver for this structure.        */

/* Data section for GD resource group statistics record               */
typedef struct {
    int  ctg_gd_chealth;    /* Gateway daemon health                  */
    char ctg_gd_cstatus[16];/* Gateway daemon status                  */
    int  ctg_gd_lallreq;    /* Number of requests processed           */
    int  ctg_gd_lluwtxnc;   /* Extended LUW transactions committed    */
    int  ctg_gd_lluwtxnr;   /* Extended LUW transactions rolled back  */
    long long ctg_gd_lruntime; /* Gateway daemon runtime (seconds)    */
    int  ctg_gd_lsynctxn;   /* Successful sync on return transactions */
    int  ctg_gd_lxatxnc;    /* XA commits successfully completed      */
    int  ctg_gd_lxatxnr;    /* XA roll backs processed                */
    char ctg_gd_sname[8];   /* Jobname of gateway daemon              */
    int  ctg_gd_iallreq;    /* Number of requests processed           */
    int  ctg_gd_iruntime;   /* Gateway daemon runtime (seconds)       */
    int  ctg_gd_isynctxn;   /* Successful sync on return transactions */
    int  ctg_gd_ixatxnc;    /* XA commits successfully completed      */
    int  ctg_gd_ixatxnr;    /* XA roll backs processed                */
    int  ctg_gd_iluwtxnc;   /* Extended LUW transactions committed    */
    int  ctg_gd_iluwtxnr;   /* Extended LUW transactions rolled back  */
    int  ctg_gd_iavresp;  /* Average Gateway daemon response time (ms)*/
    int  ctg_gd_lavresp;  /* Average Gateway daemon response time (ms)*/
    long long ctg_gd_ireqdata; /* Amount of Java client request data  */
    long long ctg_gd_lreqdata; /* Amount of Java client request data  */
    long long ctg_gd_irespdata;/* Amount of Java client response data */
    long long ctg_gd_lrespdata;/* Amount of Java client response data */
    int ctg_gd_csynctxn;    /* Number of SYNCONRETURN transactions    */
    int ctg_gd_cluwtxn;     /* Number of Extended LUW transactions    */
    int ctg_gd_cxatxn;      /* Number of XA transactions              */
    int ctg_gd_lxareq;      /* Number of XA requests                  */
    int ctg_gd_ixareq;      /* Number of XA requests                  */
    char ctg_gd_sapplid[8]; /* CICS Transaction Gateway APPLID        */
    char ctg_gd_sapplidq[8];/* CICS Transaction Gateway APPLID        */
                            /* qualifier                              */
    char ctg_gd_sstatint[6];/* Length of the statistics               */
                            /* interval HHMMSS                        */
    char ctg_gd_sstateeod[6]; /* Logical End Of Day time HHMMSS       */
    char ctg_gd_cnextreset[6];/* End of interval time HHMMSS          */
    char reserved[2];       /* Make structure length multiple of 8    */
                            /* Updates from 7.2.0.0                   */
    int ctg_gd_lhaexit;     /* Number of CICS request exit calls      */
    int ctg_gd_ihaexit;     /* Number of CICS request exit calls      */
    int ctg_gd_lsyncfail;   /* Failed SYNCONRETURN transactions       */
    int ctg_gd_isyncfail;   /* Failed SYNCONRETURN transactions       */
    char ctg_gd_sdfltsrv[8];/* Default server name                    */
    char ctg_gd_shostname[128];/* CICS TG host name                   */
    int ctg_gd_lxacomp;     /* Number of XA transactions completed    */
                            /* for HA group                           */
    int ctg_gd_ixacomp;     /* Number of XA transactions completed    */
                            /* for HA group                           */
    int ctg_gd_lxatxnhi;    /* Peak number of in flight XA            */
                            /* transactions                           */
    int ctg_gd_ixatxnhi;    /* Peak number of in flight XA            */
                            /* transactions                           */
                            /* Following 2 lines updated in 8.1.0.0   */
    int ctg_gd_lavrespio;   /* Average Gateway daemon response time   */
                            /* + I/O (ms)                             */
    int ctg_gd_iavrespio;   /* Average Gateway daemon response time   */
                            /* + I/O (ms)                             */
} GD_DATA;
#define GD_DATA_VERSION 0x00 /* ctg_recver for this structure.        */

/* Data section for WT resource group statistics record.              */
typedef struct {
    int  ctg_wt_calloc;     /* Currently allocated worker threads     */
    int  ctg_wt_ccurr;      /* Current number of worker threads       */
    int  ctg_wt_ltimeouts;  /* Number times worker timeout hit        */
    int  ctg_wt_sinit;      /* Initial number of worker threads       */
    int  ctg_wt_smax;       /* Maximum number of worker threads       */
                            /* -1 signifies no limit                  */
    int  ctg_wt_itimeouts;  /* Number of times worker timeout hit     */
    int  ctg_wt_iallochi;   /* Peak number of allocated worker threads*/
    char reserved[4];       /* Make structure length a multiple of 8  */
} WT_DATA;
#define WT_DATA_VERSION 0x00 /* ctg_recver for this structure.        */

/* Data section for SE resource group statistics record.              */
typedef struct {
    int ctg_se_selim;            /* Amount of available memory ELIM   */
    int ctg_se_celoal;           /* Amount of used memory ELOAL       */
    long long ctg_se_cheapgcmin; /* JVM heap size after GC            */
    long long ctg_se_sheapinit;  /* JVM heap size after GC            */
    long long ctg_se_sheapmax;   /* JVM maximum heap size             */
    long long ctg_se_igctime;    /* JVM GC time                       */
    long long ctg_se_lgctime;    /* JVM GC time                       */
    long long ctg_se_igccount;   /* JVM GC count                      */
    long long ctg_se_lgccount;   /* JVM GC count                      */
    int ctg_se_c31max;           /* Limit of 31-bit user memory       */
} SE_DATA;
#define SE_DATA_VERSION 0x00 /*ctg_recver for this structure.         */

/* Data section for WS resource group statistics record.              */
typedef struct {
    int  ctg_ws_creq;     /* Number of in flight web service requests */
    int  ctg_ws_cwaiting; /* Number of web service requests waiting   */
    int  ctg_ws_iallreq;  /* Number of web service requests processed */
    int  ctg_ws_iavresp;  /* Average web service response time        */
    int  ctg_ws_ireqhi;   /* Peak number of web service requests      */
    long long ctg_ws_ireqdata; /* Amount of web service request data  */
    long long ctg_ws_irespdata;/* Amount of web service response data */
    int  ctg_ws_lallreq;  /* Number of web service requests processed */
    int  ctg_ws_lavresp;  /* Average web service response time        */
    int  ctg_ws_lreqhi;   /* Peak number of web service requests      */
    long long ctg_ws_lreqdata; /* Amount of web service request data  */
    long long ctg_ws_lrespdata;/* Amount of web service response data */
    int  ctg_ws_scount;   /* Number of defined web services           */
    int  ctg_ws_smaxhttp; /* Maximum number of HTTP client threads    */
} WS_DATA;
#define WS_DATA_VERSION 0x00 /*ctg_recver for this structure.         */

/* Data section for WSx resource group statistics record.             */
typedef struct {
    char ctg_wsx_sininame[8];/* Configured web service name           */
    int  ctg_wsx_creq;    /* Number of in flight web service requests */
    int  ctg_wsx_iallreq; /* Number of web service requests processed */
    int  ctg_wsx_iavresp; /* Average web service response time        */
    int  ctg_wsx_ireqhi;  /* Peak number of web service requests      */
    long long ctg_wsx_ireqdata; /* Amount of web service request data */
    long long ctg_wsx_irespdata;/* Amount of web service response data*/
    int  ctg_wsx_lallreq; /* Number of web service requests processed */
    int  ctg_wsx_lavresp; /* Average web service response time        */
    int  ctg_wsx_lreqhi;  /* Peak number of web service requests      */
    long long ctg_wsx_lreqdata; /* Amount of web service request data */
    long long ctg_wsx_lrespdata;/* Amount of web service response data*/
    char ctg_wsx_seibtrnid[4]; /* CICS EIBTRNID transaction identifier*/
    char ctg_wsx_smirror[4];   /* CICS mirror transaction identifier  */
    char ctg_wsx_sprogram[8];  /* CICS program name                   */
    char ctg_wsx_sserver[8];   /* CICS server name                    */
    char ctg_wsx_suri[256];    /* Web service URI                     */
    char reserved[4];     /* Make structure length a multiple of 8    */
} WSx_DATA;
#define WSx_DATA_VERSION 0x00 /*ctg_recver for this structure.        */

typedef struct {
   SMF_HEADER header;
   SMF_GENERIC_PRODUCT_SECTION genProductSection;
   SMF_CTG_PRODUCT_SECTION ctgProductSection;
} SMF_START;

#pragma pack()
#endif
